function [WalkData]=Walk(s0,params,timebase,Nsteps,NoiseStep,SaveFlag,verbose)
%function that simulates multiple steps and generates walking data

%written by Jian Jin
% Copyright (c) 2022 Jian Jin
% Permission is hereby granted, free of charge, to any person obtaining a copy
% of this software and associated documentation files (the "Software"), to deal
% in the Software without restriction, including without limitation the rights
% to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
% copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

%-------------------------------------------------------------------------
%Inputs:
%s0 is initial condition with dimension 4; beta is mass ratio; gamma is slope;
%timebase is the maximal time range for each step; Nsteps is the total steps;
%NoiseStep is the noise strength for floor height;
%SaveFlag=1 means that to save aggregated data; verbose is to decide whether showing
%simulation progress

%outputs: aggregated time t and state s, heel strike index lhs and strike_interval

%beta=params.beta;
%gamma=params.gamma;
%params.delta_h=NoiseStep(Step);

WalkData.t=[];
WalkData.s=[];
% WalkData.lhs=1;

for Step=1:Nsteps
    if verbose
        disp(['busy with simulating step : ',num2str(Step)])
    end
    params.delta_h=NoiseStep(Step);%uneven floor perturbation at each step
    
    if Step>1
        [t1,s1,ie1,te1]=run_simulation_per(s0,params,timebase);
    else
        [t1,s1,ie1,te1]=run_simulation_per_pert(s0,params,timebase);
    end
    
    if ie1~=1
        if verbose
            disp(['walker has fallen at step : ',num2str(Step)])
        end
        t1_end(Step)=t1(end);
        if Step>1
            t2= sum(t1_end(1:Step-1))+t1; % sum up the all previous strike intervals to get current time
        else
            t2=t1; % no need to sum up for the first step
        end
        s1_end=s1(end,:)';
        s0=s1_end;
        %         WalkData.lhs=[WalkData.lhs lhs_index1];
        if SaveFlag==1
            WalkData.t=[WalkData.t;t2];
            WalkData.s=[WalkData.s;s1];
        end
        
        break
    end
    t1_end(Step)=t1(end);
    if Step>1
        t2= sum(t1_end(1:Step-1))+t1; % sum up the all previous strike intervals to get current time
    else
        t2=t1; % no need to sum up for the first step
    end
    
    s1_end=s1(end,:)';
    s0=s1_end;
    %     WalkData.lhs=[WalkData.lhs lhs_index1]; %get heel-strike index for all steps
    if SaveFlag==1
        WalkData.t=[WalkData.t;t2];
        WalkData.s=[WalkData.s;s1];
    end
    
end
WalkData.strike_interval=t1_end;
WalkData.fall=ie1;   %1 means not fall


