function [WalkData]=Walk_lhs(s0,params,timebase,Nsteps,NoiseStep,SaveFlag,verbose)
%function that simulates multiple steps and generates walking data

%Inputs:
%s0 is initial condition with dimension 4; beta is mass ratio; gamma is slope;
%timebase is the maximal time range for each step; Nsteps is the total steps;
%NoiseStep is the noise strength for floor height;
%SaveFlag=1 means that to save aggregated data; verbose is to decide whether showing
%simulation progress

%outputs: aggregated time t and state s, heel strike index lhs and strike_interval

%beta=params.beta;
%gamma=params.gamma;
%params.delta_h=NoiseStep(Step);

WalkData.t=[];
WalkData.s=[];
WalkData.lhs=1;

for Step=1:Nsteps
    if verbose
        disp(['busy with simulating step : ',num2str(Step)])
    end
    params.delta_h=NoiseStep(Step);%uneven floor perturbation at each step
    
    if Step>1
    [t1,s1,ie1,te1]=run_simulation_per(s0,params,timebase);
    else
    [t1,s1,ie1,te1]=run_simulation_per_pert(s0,params,timebase);           
    end
     
    if ie1~=1
        if verbose
            disp(['walker has fallen at step : ',num2str(Step)])
        end
        
        % find heelstrike index
        lhs_index1=find(t1==te1);%lhs_index1=find(abs(t1-te1)<0.0001);       
        lhs_index1=min(lhs_index1); % use min because there are two consercutive indexes
        t1_end(Step)=t1(end);
        if Step>1
            t2= sum(t1_end(1:Step-1))+t1; % sum up the all previous strike intervals to get current time
        else
            t2=t1; % no need to sum up for the first step
        end        
        s1_end=s1(end,:)';
        s0=s1_end;
        WalkData.lhs=[WalkData.lhs lhs_index1];
        if SaveFlag==1
            WalkData.t=[WalkData.t;t2];
            WalkData.s=[WalkData.s;s1];
        end
        
        break
    end
    % find heelstrike index, complete the followings later!!!
    lhs_index1=find(t1==te1);%lhs_index1=find(abs(t1-te1)<0.0001);   
    lhs_index1=min(lhs_index1); % use min because there are two consercutive indexes
    t1_end(Step)=t1(end);
    if Step>1
        t2= sum(t1_end(1:Step-1))+t1; % sum up the all previous strike intervals to get current time
    else
        t2=t1; % no need to sum up for the first step
    end
    
    s1_end=s1(end,:)';
    s0=s1_end;
    WalkData.lhs=[WalkData.lhs lhs_index1]; %get heel-strike index for all steps
    if SaveFlag==1
        WalkData.t=[WalkData.t;t2];
        WalkData.s=[WalkData.s;s1];
    end
    
end
WalkData.strike_interval=t1_end;
WalkData.fall=ie1;   %1 means not fall

%get heel strike index for a time series of multiple steps
for index=3:length(WalkData.lhs)
    WalkData.lhs(index)=WalkData.lhs(index-1)+WalkData.lhs(index)+1;% for example: if heel strike for first step happens at index 2, and heel strike for second step
    % happens at index 2, than we should have heel strike index [2 5]
end

WalkData.lhs_s=WalkData.s(WalkData.lhs,:); %states at heel-strike
WalkData.pre_impact_state=WalkData.lhs_s;

% if ie1==1
    for i=2:size(WalkData.lhs_s,1)
        % inelastic collision        
        WalkData.lhs_s(i,:)=pendulumcollision(WalkData.lhs_s(i,:),params.beta);
        WalkData.lhs_s(i,:)=WalkData.lhs_s(i,:)';
    end
% end
