function sd=Walker_Forward(t,s,params)
% Purpose: to calculate time derivatives of states, also equations of motion

%written by Dinant Kistemaker and Sjoerd Bruijn

% Copyright (c) 2022 Dinant Kistemaker and Sjoerd Bruijn
% Permission is hereby granted, free of charge, to any person obtaining a copy
% of this software and associated documentation files (the "Software"), to deal
% in the Software without restriction, including without limitation the rights
% to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
% copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

% Input: t    = time
%        s_0  = initial states
%        beta = is the mass ratio foot/hip
%        gamma= slope angle
% Output: sd  = time derrivatives
%
% S. Bruijn and D. Kistemaker
beta=params.beta;
gamma=params.gamma;

theta   = s(1); % angle of leading leg with positive y-axis that is attached to floor (i.e. has angle wrt gravity)
fi      = s(2); % angle between leading and trailing leg (hip angle)
p_theta = s(3); % generalized momentum of theta
p_fi    = s(4); % generalized momentum of fi

% Dynamics of walker using Hamiltonian dynamics
sd(1) = (2*p_theta - p_fi*(2*cos(fi) - 2))/(2*beta*sin(fi)^2 + 2);
sd(2) = (2*p_fi*(1/beta - 2*cos(fi) + 2) - p_theta*(2*cos(fi) - 2))/(2*beta*sin(fi)^2 + 2);
sd(3) = beta*sin(fi + gamma - theta) - sin(gamma - theta)*(beta + 1);
sd(4) = (4*beta*cos(fi)*sin(fi)*(p_fi^2*(1/beta - 2*cos(fi) + 2) + p_theta^2 - p_fi*p_theta*(2*cos(fi) - 2)))/(2*beta*sin(fi)^2 + 2)^2 - (2*sin(fi)*p_fi^2 + 2*p_theta*sin(fi)*p_fi)/(2*beta*sin(fi)^2 + 2) - beta*sin(fi + gamma - theta);

sd=sd';
