function [J] = jacob_traj(s,params)
%purpose: calculate the LOCAL Jacobian matrix (in Hamiltonian form) along the trajectory semi-analytically

%written by Jian Jin
% Copyright (c) 2022 Jian Jin
% Permission is hereby granted, free of charge, to any person obtaining a copy
% of this software and associated documentation files (the "Software"), to deal
% in the Software without restriction, including without limitation the rights
% to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
% copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

pert_size=1e-8;
pert=pert_size*eye(4);
Jacobian=zeros(4,4);

[s0]=xderivs(s,params);   % the only difference of function xderivs with Walker_Forward is that this function does not have input t
s0_output=s0;

for i=1:4
    s0_a=s+pert(:,i);
    [s1]=xderivs(s0_a,params);
    s0_output_a=s1;
    Jacobian(:,i)=(s0_output_a'-s0_output')/pert_size;
end

J=Jacobian;


