%% main function: generate figures
%written by Jian Jin
% Copyright (c) 2022 Jian Jin
% Permission is hereby granted, free of charge, to any person obtaining a copy
% of this software and associated documentation files (the "Software"), to deal
% in the Software without restriction, including without limitation the rights
% to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
% copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

%% Stance leg angular velocity perturbations. Robustness is maximal energy deviation
addpath(genpath('data'));
load('energy_pert_stance_push_final.mat')
figure;
p1=plot((0:0.1:3.8)./t_opt(end)*100,Hamiltonian_energy_pre_error(1:1:end,end),'b-o','MarkerSize',3,'linewidth',3);%/10
p1.MarkerSize = 4;
clear all
hold on
load('energy_pert_stance_pull_final.mat')
p2=plot((0:0.1:3.8)./t_opt(end)*100,-Hamiltonian_energy_pre_error(1:1:end,end),'r-o','MarkerSize',3,'linewidth',3);%/10
p2.MarkerSize = 4;
xlabel('percentage of single stance phase (%)');
ylabel('absolute maximal energy deviation [-]')
set(gca,'linewidth',2,'fontsize',20,'Ytick',[0:6e-4:6e-4],'Xtick',[0:50:100],'box','off')
axis([0 100 0 6e-4])
title('stance leg perturbation')
legend('push (positive energy deviation)','pull (negative energy deviation)')
set(gcf,'position',[0,0,600,480])
clear all

%% Stance leg angular velocity perturbation. Robustness is maximal angular velocity deviation (not in the manuscript)
addpath(genpath('data'));
load('energy_pert_stance_push_final.mat')
figure;
p1=plot((0:0.1:3.8)./t_opt(end)*100,phase_dependent_pert(1:1:end),'b-o','MarkerSize',3,'linewidth',2);%/10
p1.MarkerSize = 4;
clear all
hold on
load('energy_pert_stance_pull_final.mat')
p2=plot((0:0.1:3.8)./t_opt(end)*100,phase_dependent_pert(1:1:end),'r-o','MarkerSize',3,'linewidth',2);%/10
p2.MarkerSize = 4;
xlabel('percentage of single stance phase (%)');
ylabel('maximal angular velocity deviation [-]')
set(gca,'linewidth',2,'fontsize',20,'Ytick',[-0.02:0.02:0.02],'Xtick',[0:50:100],'box','off')
axis([0 100 -0.02 0.02])
title('stance leg perturbation')
legend('push','pull')
set(gcf,'position',[0,0,600,480])
clear all


%% maximal energy deviation for swing leg perturbations

figure;
subplot(4,4,1:4)
load('energy_pert_hip_pull_final.mat')%retraction perturbation
plot(3.7/t_opt(end)*100,Hamiltonian_energy_pre_error(end-1,end),'r.','MarkerSize',10,'LineWidth',2)
set(gca,'Linewidth',2,'FontSize',10,'Xtick',[0:50:100],'Ytick',[55:2:57])%
axis([0 100 55 57])
title ('swing leg perturbations')
subplot(4,4,5:16)
plot((0:0.1:3.5)./t_opt(end)*100,Hamiltonian_energy_pre_error(1:end-3,end),'r-o','MarkerSize',3,'LineWidth',2)
clear all
hold on;
load('energy_pert_hip_push_final.mat')%swing perturbation
plot((0:0.1:3.7)./t_opt(end)*100,Hamiltonian_energy_pre_error(1:end-1,end),'b-o','MarkerSize',3,'LineWidth',2)
xlabel('percentage of single stance phase (%)')
ylabel('maximal energy deviation [-]')
set(gca,'Linewidth',2,'FontSize',20,'Xtick',[0:50:100],'Ytick',[-0.0002:0.0032:0.003])%
axis([0 100 -0.0002 0.003])
set(gcf,'position',[0,0,600,480])
%% phase-dependent stability measure

s0=[0.096754232803160644671436330099823
    0.19350846560632167792093127900444
    -0.10031579185257424868815689933399
    0];%fixed point

params.beta=0.002;%fixed mass ratio
params.gamma=1e-3;%fixed slope
params.delta_h=0;%no step perturbation
timebase=[0:0.001:5];

%search for the fixed point using Newton-Raphson method
[s_star]=search(s0,params)

params.delta_h=0;  %uneven floor height, 0 means even floor
[J_star] = jacob(s_star,params);%get Jacobian matrix by numerical method
[J_star_reduced] = jacob_reduced(s_star,params);%get Jacobian matrix in reduced coordinates from the orthogonal hyperplane

[t_opt,s_opt,~,~]=run_simulation_per(s_star,params,timebase);%step function for the walker in Hamiltonian form

eig_J=[];

for j=1:(size(s_opt,1)-1)
    [J_tmp] = jacob_traj(s_opt(j,:)',params);%time-dependent Jacobian matrix over a gait cycle
    s_opt_derivs(j,:)=xderivs(s_opt(j,:)',params);
    [eigenvector_J(:,:,j),eig_J(j,:)]=eigenshuffle(J_tmp);
    eig_J_re(j,:)=abs(real(eig(J_tmp)'));  %there is a pair of equal and opposite eigenvalues with only real components
    eig_J_im(j,:)=imag(eig(J_tmp)');       %there is another conjugate pair of imaginary eigenvalues
    J_swing(:,:,j) = J_tmp;%eq 10 in Norris paper
    
end

%relative coordinates!!!
%Steps below are following Norris paper section 3.5
for j=1:(size(s_opt_derivs,1))
    U_first_row(j,:) = s_opt_derivs(j,:)'* inv(norm(s_opt_derivs(j,:)));%tangent direction
    U_first_row_two_element(j,:) = s_opt_derivs(j,1:2:3)'* inv(norm(s_opt_derivs(j,1:2:3)));%stance leg states in the tangent direction
    orth_tmp=null(U_first_row(j,:));
    orth_two_element(j,:)=null(U_first_row_two_element(j,:));%stance leg states in the orthogonal direction
    U(:,:,j) = [U_first_row(j,:);orth_tmp']';%important eigenvectors
end

dt=diff(t_opt(1:end-1));%less accuracy when using dt=mean(diff(t_opt(1:end)));
%less accuracy when using [Fx,Fy,Fz] = gradient(U,dt);
for i=1:4
    for j=1:4
        Fz(i,j,:)=diff(squeeze(U(i,j,:)))./dt;
    end
end

for j=1:(size(s_opt_derivs,1))-1
    norm_U_derivs(:,:,j)=Fz(:,:,j)*inv(U(:,:,j));
    B(:,:,j)= U(:,:,j)*J_swing(:,:,j)*inv(U(:,:,j))+norm_U_derivs(:,:,j);%eq 14 from Norris paper
    [eigenvector_B(:,:,j),eig_B(j,:)]=eigenshuffle(B(:,:,j));
end

%remove first row and column
for j=1:(size(s_opt_derivs,1))-1
    B_reduced(:,:,j) = B(2:end,2:end,j);
    [eig_b_reduced(1:3,j),idx]=sort(real(eig(B_reduced(:,:,j))));
    
end

[eigenvector_J_relative,eig_J_relative]=eigenshuffle(B_reduced(:,:,:));  % This function can be used to track sorted eigenvalues. Source: https://www.mathworks.com/matlabcentral/fileexchange/22885-eigenshuffle
eig_J_relative=real(eig_J_relative);

sum_eigen_J_relative=squeeze(sum(eig_J_relative(1:3,:))');

%plot phase-dependent stability measures
figure
a=t_opt(1:end-2)./t_opt(end-2)*100;
plot(a,eig_J_relative)
xlabel('percentage of single stance phase')
ylabel('phase-dependent stability measures')
set(0,'DefaultTextInterpreter', 'latex');
hold on
plot(a,squeeze(sum(eig_J_relative(1:3,:))'),'b','LineWidth',2)
set(gca,'Linewidth',2,'FontSize',20,'Xtick',[0:50:100],'Ytick',[-1:1:1],'box','off')
set(gcf,'position',[0,0,600,480])
% legend('eigenvalues','sum of eigenvalues')

%% correlations between phase-dependent robustness and phase-dependent stability measures

%two stability measures: trajectory-normal divergence rate and max eig
measure_divergence_rate=sum_eigen_J_relative(1:100:end);
measure_max_eigen=squeeze(max(eig_J_relative(1:3,1:100:end))');

clearvars -except measure_divergence_rate measure_max_eigen

% stance leg perturbations, trajectory-normal divergence rate
figure;
load('energy_pert_stance_pull_final.mat')
Hamiltonian_energy_pre_error_plus=Hamiltonian_energy_pre_error;
plot(measure_divergence_rate,Hamiltonian_energy_pre_error_plus(:,end),'ro','MarkerSize',3,'LineWidth',2)
[rho1pull,pval1pull]=corr(measure_divergence_rate,Hamiltonian_energy_pre_error_plus(:,end),'type','Kendall');
rho1pull
clearvars -except measure_divergence_rate measure_max_eigen Hamiltonian_energy_pre_error_plus rho1pull pval1pull
load('energy_pert_stance_push_final.mat')
hold on
Hamiltonian_energy_pre_error_minus=Hamiltonian_energy_pre_error;
plot(measure_divergence_rate,Hamiltonian_energy_pre_error_minus(:,end),'bo','MarkerSize',3,'LineWidth',2)
[rho1push,pval1push]=corr(measure_divergence_rate,Hamiltonian_energy_pre_error_minus(:,end),'type','Kendall');
rho1push
hold on
plot(measure_divergence_rate,Hamiltonian_energy_pre_error_minus(:,end)-Hamiltonian_energy_pre_error_plus(:,end),'mo','MarkerSize',3,'LineWidth',2)
[rho1sum,pval1sum]=corr(measure_divergence_rate,Hamiltonian_energy_pre_error_minus(:,end)-Hamiltonian_energy_pre_error_plus(:,end),'type','Kendall');
rho1sum
xlabel('trajectory-normal divergence rate')
ylabel('maximal energy deviation [-]')
legend('pull','push','sum')
set(gca,'Linewidth',2,'FontSize',20,'Xtick',[-1:1:1],'Ytick',[-5e-4:5e-4:10e-4],'box','off')%
title ('stance leg perturbation')
axis([-1 1 -5e-4 10e-4])
set(gcf,'position',[0,0,600,480])

% stance leg pert,maximal eigenvalue,
figure
% load('energy_pert_plus6.mat')
plot(measure_max_eigen,Hamiltonian_energy_pre_error_plus(:,end),'ro','MarkerSize',3,'LineWidth',2)
[rho2pull,pval2pull]=corr(measure_max_eigen,Hamiltonian_energy_pre_error_plus(:,end),'type','Kendall');
rho2pull
hold on
plot(measure_max_eigen,Hamiltonian_energy_pre_error_minus(:,end),'bo','MarkerSize',3,'LineWidth',2)
[rho2push,pval2push]=corr(measure_max_eigen,Hamiltonian_energy_pre_error_minus(:,end),'type','Kendall');
rho2push
hold on
plot(measure_max_eigen,Hamiltonian_energy_pre_error_minus(:,end)-Hamiltonian_energy_pre_error_plus(:,end),'mo','MarkerSize',3,'LineWidth',2)
[rho2sum,pval2sum]=corr(measure_max_eigen,Hamiltonian_energy_pre_error_minus(:,end)-Hamiltonian_energy_pre_error_plus(:,end),'type','Kendall');
rho2sum
xlabel('maximal eigenvalue')
ylabel('maximal energy deviation [-]')
% set(gca,'Linewidth',2,'FontSize',20)%,'Xtick',[0:50:100],'Ytick',[-1:1:1]
title ('stance leg perturbation')
legend('pull','push','sum')
set(gca,'Linewidth',2,'FontSize',20,'Xtick',[-0.01:1.01:1],'Ytick',[-5e-4:5e-4:10e-4],'box','off')%
axis([-0.01 1 -5e-4 10e-4])
set(gcf,'position',[0,0,600,480])

clearvars -except measure_divergence_rate measure_max_eigen

%swing leg perturbations and trajectory-normal divergence rate
figure;
subplot(4,4,1:4)
load('energy_pert_hip_pull_final.mat')
Hamiltonian_energy_pre_error_plus=Hamiltonian_energy_pre_error;
load('energy_pert_hip_push_final.mat')
Hamiltonian_energy_pre_error_minus=Hamiltonian_energy_pre_error;
plot(measure_divergence_rate(end-1),Hamiltonian_energy_pre_error_plus(end-1,end),'ro','MarkerSize',3,'LineWidth',2)
set(gca,'Linewidth',2,'FontSize',12,'Xtick',[-1:1:1],'Ytick',[55:2:57])%
axis([-1 1 55 57])
title ('swing leg perturbation')
subplot(4,4,5:16)
plot(measure_divergence_rate(1:end-2),Hamiltonian_energy_pre_error_plus(1:end-2,end),'ro','MarkerSize',3,'LineWidth',2)
[rho3pull,pval3pull]=corr(measure_divergence_rate(1:end-1),Hamiltonian_energy_pre_error_plus(1:end-1,end),'type','Kendall');
rho3pull
hold on;
plot(measure_divergence_rate(1:end-1),Hamiltonian_energy_pre_error_minus(1:end-1,end),'bo','MarkerSize',3,'LineWidth',2)
[rho3push,pval3push]=corr(measure_divergence_rate(1:end-1),Hamiltonian_energy_pre_error_minus(1:end-1,end),'type','Kendall');
rho3push
xlabel('trajectory-normal divergence rate')
ylabel('maximal energy deviation [-]')
set(gca,'Linewidth',2,'FontSize',12,'Xtick',[-1:1:1],'Ytick',[-0.0002:0.0032:0.003])%
axis([-1 1 -0.0002 0.003])
set(gcf,'position',[0,0,600,480])

%swing leg perturbations and maximal eigenvalue
figure;
% load('energy_pert_hip_push_final.mat')
subplot(4,4,1:4)
plot(measure_max_eigen(end-1),Hamiltonian_energy_pre_error_plus(end-1,end),'ro','MarkerSize',3,'LineWidth',2)
set(gca,'Linewidth',2,'FontSize',12,'Xtick',[-0.01:1.01:1],'Ytick',[55:2:57])%
axis([-0.01 1 55 57])
title ('swing leg perturbation')
subplot(4,4,5:16)
plot(measure_max_eigen(1:end-2),Hamiltonian_energy_pre_error_plus(1:end-2,end),'ro','MarkerSize',3,'LineWidth',2)
[rho4pull,pval4pull]=corr(measure_max_eigen(1:end-1),Hamiltonian_energy_pre_error_plus(1:end-1,end),'type','Kendall');
rho4pull
hold on;
plot(measure_max_eigen(1:end-1),Hamiltonian_energy_pre_error_minus(1:end-1,end),'bo','MarkerSize',3,'LineWidth',2)
[rho4push,pval4push]=corr(measure_max_eigen(1:end-1),Hamiltonian_energy_pre_error_minus(1:end-1,end),'type','Kendall');
rho4push
xlabel('maximal eigenvalue')
ylabel('maximal energy deviation')
set(gca,'Linewidth',2,'FontSize',12,'Xtick',[-0.01:1.01:1],'Ytick',[-0.0002:0.0032:0.003])%
axis([-0.01 1 -0.0002 0.003])
set(gcf,'position',[0,0,600,480])
