%% script purpose: generate phase-dependent robustness for stance leg perturbations

%written by Jian Jin
% Copyright (c) 2022 Jian Jin
% Permission is hereby granted, free of charge, to any person obtaining a copy
% of this software and associated documentation files (the "Software"), to deal
% in the Software without restriction, including without limitation the rights
% to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
% copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

clear all
clc

addpath(genpath('data'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%select here push or pull pert
pert_direction=1;  %pull
% pert_direction=-1;  %push
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

params.pert_duration=0;%perturbation duration, set to 0 to indicate instaneous

%step size/accuracy of the magnitude of the applied perturbations
step_size1=0.01;
step_size2=0.001;
step_size3=0.0001;

pert_save=[];

t1=[];
s1=[];
s_end=[];
total_step=0;
s0_initial=[0.096754232803160644671436330099823
    0.19350846560632167792093127900444
    -0.10031579185257424868815689933399
    0];% fixed point

params.beta=0.002;%fixed mass ratio
params.gamma=1e-3;%fixed slope

Nsteps=50;% test whether falling within 50 steps
NoiseStep=zeros(Nsteps,1)*0;% step-up and step-down perturbations set to 0
params.delta_h=0;% step-up and step-down perturbations set to 0

%load fixed point data,choose first row first column as fix slope/mass ratio
load SstarAll_1_dense_red.mat

%initialize robustness
params.push_pert=0;

%nominal total energy
Hamiltonian_energy_ref = (1+params.beta)*cos(params.gamma-s0_initial(1))-params.beta*cos(params.gamma-s0_initial(1)+s0_initial(2))+(s0_initial(3).^2+(2-2*cos(s0_initial(2))).*s0_initial(3).*s0_initial(4)+(2-2*cos(s0_initial(2))+params.beta^-1).*s0_initial(4).^2)./(2+2*params.beta*sin(s0_initial(2).^2));

for i_gamma=1%fixed slope
    for i_beta=1%fixed mass ratio
        params.time_window=0;
        
        t=[];
        s=[];
        t_tmp=[];
        stack_J_eig=[];
        PHIdot=[];
        s_star =squeeze(s_star_save(i_beta,i_gamma,:));
        beta=beta_save(i_beta,i_gamma);
        gamma=gamma_save(i_beta,i_gamma);
        params.beta=beta;
        params.gamma=gamma;
        
        timebase=[0:0.1:20];
        SaveFlag=1;%1 means save concatenated states, which will slower time;
        verbose=0; %0 means not showing "simulating step"
        
        [t_opt,s_opt,~,~]=run_simulation_per(s_star,params,timebase);%step function for the walker in Hamiltonian form
        
        NoiseStep=zeros(Nsteps,1);% step-up and step-down pert set to 0
        
        [WalkData2]=Walk(s0_initial,params,timebase,Nsteps,NoiseStep,SaveFlag,verbose);  %Walk
        
        for index=1:size(s_opt,1)
            mass_matrix(:,:,index)=norris_mass_matrix(s_opt(index,:),params);%mass matrix, which maps from angular velocity to generalized momentum
        end
        
        for i_timeindex=1:39%perturbations added at consecutive points in the gait cycle separated by a normalized time of 0.1
            i_timeindex
            s0=s_opt(i_timeindex,:);%use state at particular phase of the periodic orbit as initial state
            
            s_opt_derivs(i_timeindex,:)=xderivs(s0',params);%state derivatives
            
            U_first_row(i_timeindex,:) = s_opt_derivs(i_timeindex,:)'* inv(norm(s_opt_derivs(i_timeindex,:)));% tangent direction
            U_two_elements(i_timeindex,:) = [U_first_row(i_timeindex,1);U_first_row(i_timeindex,3)];%stance leg states in the tangent direction
            
            %computing angular velocities for stance leg angle and hip angle (nominnal)
            angular_velocities(:,i_timeindex)=inv(mass_matrix(:,:,i_timeindex))*[s0(3);s0(4)];
            
            orth_tmp=null(U_two_elements(i_timeindex,:));%stance leg states in the orthogonal direction
            pert_tmp(1:4)=0; %this temporary variable saves the maximum allowable perturbation, larger columns have higher accuracy
            
            if i_beta==1
                if i_gamma~=1
                    pert_tmp(1)=pert_save(i_beta,i_gamma-1,:);
                end
            elseif i_gamma==1
                if i_beta~=1
                    pert_tmp(1)=pert_save(i_beta-1,i_gamma,:);
                end
            end
            if i_beta~=1 && i_gamma~=1
                pert_tmp(1)=pert_save(i_beta-1,i_gamma,:);
            end
            pert_tmp(2)=pert_tmp(1);
            %check if you fall for this pert magnitude
            %% if it did not fall
            if WalkData2.fall==1 %1 means not falling
                disp('I did not fall in previous step, and will now increase step height')
                
                %increase perturbation magnitude, starting from the first
                %accuracy digit
                for pert_mag=pert_tmp(1):step_size1:pert_tmp(1)+step_size1*10
                    NoiseStep=[zeros(Nsteps,1)]; %zero step-up or step-down perturbation
                    %params.push_pert=pert_mag*pert_direction;
                    %s0_pert_two_element=[s0(1);s0(3)]+orth_tmp*pert_mag*pert_direction;
                    
                    %perturbation is on the stance leg angular velocity
                    perturbed_angular_velocities(:,i_timeindex)=[angular_velocities(1,i_timeindex)+pert_mag*pert_direction;angular_velocities(2,i_timeindex)];
                    
                    %transofrm the perturbed states of angular velocities to states of generalized momenta
                    perturbed_momenta(:,i_timeindex)=mass_matrix(:,:,i_timeindex)*perturbed_angular_velocities(:,i_timeindex);
                    
                    s0_pert=[s0(1) s0(2) perturbed_momenta(1,i_timeindex) perturbed_momenta(2,i_timeindex)]';
                    %                     s0_pert=[s0_pert_two_element(1) s0(2) s0_pert_two_element(2) s0(4)]';
                    [WalkData]=Walk(s0_pert,params,timebase,Nsteps,NoiseStep,SaveFlag,verbose);
                    
                    if WalkData.fall~=1
                        if pert_mag==pert_tmp(1)
                            pert_tmp(2)=max(0,pert_mag-step_size1);
                            pert_tmp(3)=max(0,pert_mag-step_size1);
                            pert_tmp(4)=max(0,pert_mag-step_size1);
                        end
                        break
                    end
                    
                    %save the temporary maximum allowable perturbation to all
                    %columns of pert_tmp, and increase accuracy in later
                    %columns
                    pert_tmp(2)=pert_mag;
                    pert_tmp(3)=pert_mag;
                    pert_tmp(4)=pert_mag;
                    clear WalkData
                end
                for pert_mag=pert_tmp(2):step_size2:pert_tmp(2)+step_size2*10
                    NoiseStep=zeros(Nsteps,1);
                    %params.push_pert=pert_mag*pert_direction;
                    %s0_pert_two_element=[s0(1);s0(3)]+orth_tmp*pert_mag*pert_direction;
                    %                     s0_pert=[s0_pert_two_element(1) s0(2) s0_pert_two_element(2) s0(4)]';
                    perturbed_angular_velocities(:,i_timeindex)=[angular_velocities(1,i_timeindex)+pert_mag*pert_direction;angular_velocities(2,i_timeindex)];
                    perturbed_momenta(:,i_timeindex)=mass_matrix(:,:,i_timeindex)*perturbed_angular_velocities(:,i_timeindex);
                    
                    s0_pert=[s0(1) s0(2) perturbed_momenta(1,i_timeindex) perturbed_momenta(2,i_timeindex)]';
                    
                    [WalkData]=Walk(s0_pert,params,timebase,Nsteps,NoiseStep,SaveFlag,verbose);
                    
                    if WalkData.fall~=1   %fall
                        break
                    end
                    
                    pert_tmp(3)=pert_mag;
                    pert_tmp(4)=pert_mag;
                    clear WalkData
                end
                %if not fall in the second accuracy digits, increase values in
                %the last accuracy digits
                for pert_mag=pert_tmp(3):step_size3:pert_tmp(3)+step_size3*10
                    NoiseStep=zeros(Nsteps,1);
                    %params.push_pert=pert_mag*pert_direction;
                    %s0_pert_two_element=[s0(1);s0(3)]+orth_tmp*pert_mag*pert_direction;
                    
                    perturbed_angular_velocities(:,i_timeindex)=[angular_velocities(1,i_timeindex)+pert_mag*pert_direction;angular_velocities(2,i_timeindex)];
                    perturbed_momenta(:,i_timeindex)=mass_matrix(:,:,i_timeindex)*perturbed_angular_velocities(:,i_timeindex);
                    
                    s0_pert=[s0(1) s0(2) perturbed_momenta(1,i_timeindex) perturbed_momenta(2,i_timeindex)]';
                    
                    %                     s0_pert=[s0_pert_two_element(1) s0(2) s0_pert_two_element(2) s0(4)]';
                    %                     clear t1 s1 s_end
                    
                    [WalkData]=Walk(s0_pert,params,timebase,Nsteps,NoiseStep,SaveFlag,verbose);
                    
                    if WalkData.fall~=1
                        clear WalkData %s_end
                        pert_mag_update=pert_mag-step_size3;
                        %s0_pert_two_element=[s0(1);s0(3)]+orth_tmp*pert_mag_update*pert_direction;
                        
                        perturbed_angular_velocities(:,i_timeindex)=[angular_velocities(1,i_timeindex)+pert_mag*pert_direction;angular_velocities(2,i_timeindex)];
                        perturbed_momenta(:,i_timeindex)=mass_matrix(:,:,i_timeindex)*perturbed_angular_velocities(:,i_timeindex);
                        
                        s0_pert=[s0(1) s0(2) perturbed_momenta(1,i_timeindex) perturbed_momenta(2,i_timeindex)]';
                        
                        %                         s0_pert=[s0_pert_two_element(1) s0(2) s0_pert_two_element(2) s0(4)]';
                        break
                    end
                    
                    pert_tmp(4)=pert_mag;
                    final_WalkData=WalkData;
                end
                %% if it falls
            elseif WalkData2.fall~=1 %means fall
                disp('oops, I fall using the previous step height, will now decrease')
                
                %decrease perturbation magnitude, starting from the first
                %accuracy digit
                for pert_mag=pert_tmp(1):-step_size1:max(pert_tmp(1)-step_size1*10,0)
                    NoiseStep=zeros(Nsteps,1); %normal distribution
                    %params.push_pert=pert_mag*pert_direction;
                    %s0_pert_two_element=[s0(1);s0(3)]+orth_tmp*pert_mag*pert_direction;
                    %                     s0_pert=[s0_pert_two_element(1) s0(2) s0_pert_two_element(2) s0(4)]';
                    perturbed_angular_velocities(:,i_timeindex)=[angular_velocities(1,i_timeindex)+pert_mag*pert_direction;angular_velocities(2,i_timeindex)];
                    perturbed_momenta(:,i_timeindex)=mass_matrix(:,:,i_timeindex)*perturbed_angular_velocities(:,i_timeindex);
                    
                    s0_pert=[s0(1) s0(2) perturbed_momenta(1,i_timeindex) perturbed_momenta(2,i_timeindex)]';
                    
                    [WalkData]=Walk(s0_pert,params,timebase,Nsteps,NoiseStep,SaveFlag,verbose);
                    
                    if WalkData.fall==1
                        pert_tmp(2)=pert_mag;
                        pert_tmp(3)=pert_mag;
                        pert_tmp(4)=pert_mag;
                        break
                    end
                    
                    pert_tmp(2)=0;
                    pert_tmp(3)=0;
                    pert_tmp(4)=0;
                    clear WalkData
                end
                
                for pert_mag=pert_tmp(2):step_size2:pert_tmp(2)+step_size2*10
                    NoiseStep=zeros(Nsteps,1);
                    %params.push_pert=pert_mag*pert_direction;
                    %s0_pert_two_element=[s0(1);s0(3)]+orth_tmp*pert_mag*pert_direction;
                    %                     s0_pert=[s0_pert_two_element(1) s0(2) s0_pert_two_element(2) s0(4)]';
                    perturbed_angular_velocities(:,i_timeindex)=[angular_velocities(1,i_timeindex)+pert_mag*pert_direction;angular_velocities(2,i_timeindex)];
                    perturbed_momenta(:,i_timeindex)=mass_matrix(:,:,i_timeindex)*perturbed_angular_velocities(:,i_timeindex);
                    
                    s0_pert=[s0(1) s0(2) perturbed_momenta(1,i_timeindex) perturbed_momenta(2,i_timeindex)]';
                    
                    [WalkData]=Walk(s0_pert,params,timebase,Nsteps,NoiseStep,SaveFlag,verbose);
                    
                    if WalkData.fall~=1
                        break
                    end
                    
                    pert_tmp(3)=pert_mag;
                    pert_tmp(4)=pert_mag;
                    clear WalkData
                    
                end
                
                for pert_mag=pert_tmp(3):step_size3:pert_tmp(3)+step_size3*10
                    NoiseStep=zeros(Nsteps,1); %no step pert
                    %s0_pert_two_element=[s0(1);s0(3)]+orth_tmp*pert_mag*pert_direction;
                    %                     s0_pert=[s0_pert_two_element(1) s0(2) s0_pert_two_element(2) s0(4)]';
                    perturbed_angular_velocities(:,i_timeindex)=[angular_velocities(1,i_timeindex)+pert_mag*pert_direction;angular_velocities(2,i_timeindex)];
                    perturbed_momenta(:,i_timeindex)=mass_matrix(:,:,i_timeindex)*perturbed_angular_velocities(:,i_timeindex);
                    
                    s0_pert=[s0(1) s0(2) perturbed_momenta(1,i_timeindex) perturbed_momenta(2,i_timeindex)]';
                    
                    [WalkData]=Walk(s0_pert,params,timebase,Nsteps,NoiseStep,SaveFlag,verbose);
                    
                    if WalkData.fall~=1
                        clear WalkData
                        pert_mag_update=pert_mag-step_size3;
                        %s0_pert_two_element=[s0(1);s0(3)]+orth_tmp*pert_mag_update*pert_direction;
                        %                         s0_pert=[s0_pert_two_element(1) s0(2) s0_pert_two_element(2) s0(4)]';
                        perturbed_angular_velocities(:,i_timeindex)=[angular_velocities(1,i_timeindex)+pert_mag*pert_direction;angular_velocities(2,i_timeindex)];
                        perturbed_momenta(:,i_timeindex)=mass_matrix(:,:,i_timeindex)*perturbed_angular_velocities(:,i_timeindex);
                        
                        s0_pert=[s0(1) s0(2) perturbed_momenta(1,i_timeindex) perturbed_momenta(2,i_timeindex)]';
                        
                        break
                    end
                    
                    pert_tmp(4)=pert_mag;
                    final_WalkData=WalkData;
                end
            end
            
            %the last column of pert_save has the most accurate maximum
            %allowable perturbation
            pert_save(i_beta,i_gamma)=pert_tmp(4)*pert_direction%accurate robustness
            total_step=total_step+1;
            
            phase_dependent_pert(i_timeindex)=pert_save(i_beta,i_gamma)%save robustness as a function of phase
            
            s0_grid=s0'+(s0_pert-s0')*0.1*(0:10);%after finding this largest robustness, also checking 0,10%,20%,... magnitude of the largest robustness, see results below
            for grid_index=1:11
                s1=[];
                [t1,s1,ie1,~]=run_simulation_per(s0_grid(:,grid_index),params,timebase);
                
                %total energy just before collision
                Hamiltonian_energy_pre(i_timeindex,grid_index) = (1+params.beta)*cos(params.gamma-s1(end-1,1))-params.beta*cos(params.gamma-s1(end-1,1)+s1(end-1,2))+(s1(end-1,3).^2+(2-2*cos(s1(end-1,2))).*s1(end-1,3).*s1(end-1,4)+(2-2*cos(s1(end-1,2))+params.beta^-1).*s1(end-1,4).^2)./(2+2*params.beta*sin(s1(end-1,2).^2));
                Hamiltonian_energy_pre_error(i_timeindex,grid_index)=Hamiltonian_energy_pre(i_timeindex,grid_index)-Hamiltonian_energy_ref;
                
                %total energy just after collision
                Hamiltonian_energy_post(i_timeindex,grid_index) = (1+params.beta)*cos(params.gamma-s1(end,1))-params.beta*cos(params.gamma-s1(end,1)+s1(end,2))+(s1(end,3).^2+(2-2*cos(s1(end,2))).*s1(end,3).*s1(end,4)+(2-2*cos(s1(end,2))+params.beta^-1).*s1(end,4).^2)./(2+2*params.beta*sin(s1(end,2).^2));
                Hamiltonian_energy_post_error(i_timeindex,grid_index)= Hamiltonian_energy_post(i_timeindex,grid_index)-Hamiltonian_energy_ref;
                s0_pert_grid(i_timeindex,grid_index,:)=s0_grid(:,grid_index);
                s0_pert_derivs(i_timeindex,grid_index,:)=xderivs(s0_grid(:,grid_index),params);
                return_rate(i_timeindex,grid_index)=dot([s0_pert_derivs(i_timeindex,grid_index,1) s0_pert_derivs(i_timeindex,grid_index,3)],orth_tmp);
                pre_collision_state(i_timeindex,grid_index,:)=s1(end-1,:);
                post_collision_state(i_timeindex,grid_index,:)=s1(end,:);
                fall(i_timeindex,grid_index)=ie1;% assess falling or not falling
                step_time(i_timeindex,grid_index)=t1(end);
                collision_measure(i_timeindex,grid_index)=abs(cos(s1(end,2)))./(1+params.beta*sin(s1(end,2)).^2);%divergence of foot strike measure
                clear s1 t1 ie1
            end
            sensitivity(i_timeindex)=Hamiltonian_energy_post_error(i_timeindex,2)/Hamiltonian_energy_pre_error(i_timeindex,2);
            robustness_energy_error(i_timeindex)=Hamiltonian_energy_pre_error(i_timeindex,end);%robustness in terms of energy error from the unperturbed energy
            
            
            
        end
        
    end
end
%save results
if pert_direction==1
    save('energy_pert_stance_pull_final.mat')
else
    save('energy_pert_stance_push_final.mat')
end