function [s_star] = search(s0,params)
%purpose: to find period-one solution using Newton-Raphson method

%written by Jian Jin
% Copyright (c) 2022 Jian Jin
% Permission is hereby granted, free of charge, to any person obtaining a copy
% of this software and associated documentation files (the "Software"), to deal
% in the Software without restriction, including without limitation the rights
% to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
% copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

maxiter=500;
s=s0; % initial guessing

for i=1:maxiter
    [t1,s1]=run_simulation_per(s,params,[0 5]);%step function
    c=s1(end,:);
    J=jacob(s,params);
    deltas=(eye(4)-J)\(c'-s);
    s=s+deltas;
    
    if norm(deltas)<.0000000001 %convergence condition
        break
    end
    
end

s_star=s;
