function xdot = xderivs(x,params)
%purpose:find xdot analytically in Hamiltonian form, the only difference between this function
%and Walker_Forward is that this one does not have input t

%written by Dinant Kistemaker and Sjoerd Bruijn
% Copyright (c) 2022 Dinant Kistemaker and Sjoerd Bruijn
% Permission is hereby granted, free of charge, to any person obtaining a copy
% of this software and associated documentation files (the "Software"), to deal
% in the Software without restriction, including without limitation the rights
% to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
% copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

beta=params.beta;
gamma=params.gamma;

theta   = x(1); % angle of leading leg with positive y-axis that is attached to floor (i.e. has angle wrt gravity)
fi      = x(2); % angle between leading and trailing leg (hip angle)
p_theta = x(3); % generalized momentum of theta
p_fi    = x(4); % generalized momentum of fi

% Dynamics of walker using Hamiltonian dynamics
xdot(1) = (2*p_theta - p_fi*(2*cos(fi) - 2))/(2*beta*sin(fi)^2 + 2);
xdot(2) = (2*p_fi*(1/beta - 2*cos(fi) + 2) - p_theta*(2*cos(fi) - 2))/(2*beta*sin(fi)^2 + 2);
xdot(3) = beta*sin(fi + gamma - theta) - sin(gamma - theta)*(beta + 1);
xdot(4) = (4*beta*cos(fi)*sin(fi)*(p_fi^2*(1/beta - 2*cos(fi) + 2) + p_theta^2 - p_fi*p_theta*(2*cos(fi) - 2)))/(2*beta*sin(fi)^2 + 2)^2 - (2*sin(fi)*p_fi^2 + 2*p_theta*sin(fi)*p_fi)/(2*beta*sin(fi)^2 + 2) - beta*sin(fi + gamma - theta);

xdot=xdot';
